#ifndef SRC_TIME_STEPPING_ADAPTIVETIMESTEPPER_HH
#define SRC_TIME_STEPPING_ADAPTIVETIMESTEPPER_HH

#include <fstream>

#include "coupledtimestepper.hh"

struct IterationRegister {
  void registerCount(FixedPointIterationCounter count);
  void registerFinalCount(FixedPointIterationCounter count);

  void reset();

  FixedPointIterationCounter totalCount;
  FixedPointIterationCounter finalCount;
};

template <class Factory, class ContactNetwork, class Updaters, class ErrorNorms>
class AdaptiveTimeStepper {
  struct UpdatersWithCount {
    Updaters updaters;
    FixedPointIterationCounter count;
  };

  using Vector = typename Factory::Vector;
  using IgnoreVector = typename Factory::BitVector;
  //using ConvexProblem = typename Factory::ConvexProblem;
  //using Nonlinearity = typename Factory::Nonlinearity;

  using MyCoupledTimeStepper = CoupledTimeStepper<Factory, ContactNetwork, Updaters, ErrorNorms>;

  using GlobalFriction = typename MyCoupledTimeStepper::GlobalFriction;
  using BitVector = typename MyCoupledTimeStepper::BitVector;
  using ExternalForces = typename MyCoupledTimeStepper::ExternalForces;

public:
  AdaptiveTimeStepper(
                      Dune::ParameterTree const &parset,
                      const ContactNetwork& contactNetwork,
                      const IgnoreVector& ignoreNodes,
                      GlobalFriction& globalFriction,
                      const std::vector<const BitVector*>& bodywiseNonmortarBoundaries,
                      Updaters &current,
                      double relativeTime,
                      double relativeTau,
                      ExternalForces& externalForces,
                      const ErrorNorms& errorNorms,
                      std::function<bool(Updaters &, Updaters &)> mustRefine);

  bool reachedEnd();

  IterationRegister advance();

  double relativeTime_;
  double relativeTau_;

private:
  UpdatersWithCount step(Updaters const &oldUpdaters, double rTime,
                         double rTau);

  double finalTime_;
  Dune::ParameterTree const &parset_;
  const ContactNetwork& contactNetwork_;
  const IgnoreVector& ignoreNodes_;

  GlobalFriction& globalFriction_;
  const std::vector<const BitVector*>& bodywiseNonmortarBoundaries_;

  Updaters &current_;
  UpdatersWithCount R1_;
  ExternalForces& externalForces_;
  std::function<bool(Updaters &, Updaters &)> mustRefine_;
  const ErrorNorms& errorNorms_;

  IterationRegister iterationRegister_;
};
#endif
